from cgi import parse_qs
from lxml.etree import parse
from meresco.components.http import utils as httputils
from meresco.framework import Observable
from urlparse import urlsplit

namespaceMap = {
    'mods':"http://www.loc.gov/mods/v3",
    'extra':"http://meresco.com/namespace/fields/extra",
    'meta':"http://meresco.com/namespace/harvester/meta",
}

def xpath(node, path):
    result = node.xpath(path, namespaceMap)
    return result and result[0] or ''

def nodeset(node, path):
    return node.xpath(path, namespaceMap)

def add(aDictionary, key, value):
    if value:
            aDictionary[key] = value

class Record(object):
    def createRecord(self, metadata):
       self.metadata = metadata
       self.record = {}
       add(self.record, 'genre', self._getGenre())
       add(self.record, 'authors', self._getNames('aut'))
       add(self.record, 'supervisors', self._getNames('ths'))
       add(self.record, 'title', self._getTitle())
       add(self.record, 'date', self._getDate())
       add(self.record, 'source', self._getSource())
       add(self.record, 'abstract', self._getAbstract())
       add(self.record, 'url', self._getUrl())
       add(self.record, 'publisher', self._getPublisher())
       add(self.record, 'keywords', self._getKeywords())
       add(self.record, 'classification', self._getClassification())
       self._handleIdentifiers()
       return self.record

    def getMods(self, metadata):
        return xpath(metadata, '//mods:mods')

    def _getNames(self, role):
        names = ""
	family = ""
	given = ""
        for name in nodeset(self.metadata, '//mods:mods/mods:name'):
            if xpath(name, 'mods:role/mods:roleTerm/text()') == role:
                for namePart in nodeset(name, 'mods:namePart'):
		    type = xpath(namePart, '@type')
                    if type == 'family' or not type:
		        family = xpath(namePart, 'text()')
                    elif type == 'given':  given  = xpath(namePart, 'text()')
                if family: names += family
                if given: names += ', ' + given
		if family: names += '; '
		family = ""
		given = ""
        if role == 'eds': return names.rstrip('; ') + ' (Ed.)'
        else: return names.rstrip(' ;')

    def _getTitle(self):
        title = xpath(self.metadata, '//mods:mods/mods:titleInfo/mods:title/text()') or '[NO TITLE AVAILABLE]'
        return title

    def _getGenre(self):
        genre = xpath(self.metadata, '//mods:mods/mods:genre/text()')
        if genre and '/' in genre: genre = genre.split('/')[-1]
	return genre

    def _getDate(self):
        return xpath(self.metadata, '//mods:originInfo/mods:dateIssued/text()')[0:4]

    def _getHostTitle(self):
        return xpath(self.metadata, '//mods:mods/mods:relatedItem/mods:titleInfo/mods:title/text()')

    def _getHostEditors(self):
        editors = ""
	family = ""
	given = ""
        for name in nodeset(self.metadata, '//mods:mods/mods:relatedItem/mods:name'):
            if xpath(name, 'mods:role/mods:roleTerm/text()') == 'edt':
                for namePart in nodeset(name, 'mods:namePart'):
                    type = xpath(namePart, '@type')
                    if type == 'family': family = xpath(namePart, 'text()')
                    elif type == 'given':  given = xpath(namePart, 'text()')
                if family: editors += family
                if given:  editors += ', ' + given
		if family: editors += '; '
		family = ""
		given = ""
        if editors: return editors.rstrip('; ') + ' (Ed.)'
	else: return ''
        
    def _getSource(self):
        editors = self._getHostEditors()
	title = self._getHostTitle()
	if editors: return editors + ' ' + '<i>' + self._getHostTitle() + '</i>'
	else: return title

    def _getAbstract(self):
        return xpath(self.metadata, '//mods:abstract/text()')

    def _getUrl(self):
        return xpath(self.metadata, '//mods:location/mods:url/text()')

    def _getPublisher(self):
        return xpath(self.metadata, '//mods:publisher/text()')

    def _getKeywords(self):
        keywords = ""
        for topic in nodeset(self.metadata, '//mods:mods/mods:subject/mods:topic'):
            keywords += xpath(topic, 'text()') + '; '
	if keywords: return keywords.rstrip('; ')
	else: return ''

    def _getClassification(self):
        string = ""
	for classification in nodeset(self.metadata, '//mods:mods/mods:classification'):
            string += xpath(classification, 'text()') + '; '
	return string.rstrip('; ')

    def _handleIdentifiers(self):
	identifiers = (nodeset(self.metadata, '//mods:identifier/text()'))
	if not identifiers: return
	for id in identifiers:
            idParts = id.split(':')
            ### urn:{issn|issbn}:<value>
            if len(idParts) < 3: break
            if idParts[1] == 'issn' or idParts[1] == 'ISSN':
                add(self.record, 'issn', idParts[2])
            if idParts[1] == 'isbn' or idParts == 'ISBN':
                add(self.record, 'isbn', idParts[2])

class RecordXml(Observable):
    def __init__(self):
         Observable.__init__(self)

    def _field(self, label, value):
       field = '\n <field><label>%s</label><content>%s</content></field>' %(label, value)
       return field

    def handleRequest(self, RequestURI='', **kwargs):
####### TP: toevoegen het afvangen van exceptions: 1. er is geen record id, 
####### 2. er wordt geen metadata gevonden
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        arguments = parse_qs(Query)
####### TP: de volgende regels staan ook in html/dynamic/resultlist.sf en
####### in html/dynamic/recordview.sf. Op 3 plaatsen dezelfde code bijhouden is slecht.
        recordId = arguments.get('recordId', [''])[0]
        if recordId.split(':')[0] in ['opc-uvt-nl']:
            metadata = parse(self.any.getStream(recordId, 'mods'))
        else:
            metadata = parse(self.any.getStream(recordId, 'metadata'))
        record = Record().createRecord(metadata)
        yield httputils.okXml
        yield """<?xml version="1.0" encoding="UTF-8"?>
<record>"""
####### TP: zeer vergelijkbare code ook in html/dynamic/recordview.sf -->
####### alles naar **een** plek
####### TP: labels zijn in het Engels. Toevoegen ondersteunen van andere talen.
####### TP: ook handig is om per field het type toe te voegen (taalonafhankelijk)
        if ('title' in record): yield self._field('Title', record['title'])
        if ('authors' in record): yield self._field('Author(s)', record['authors'])
        if ('supervisors' in record): yield self._field('Supervisor(s)', record['supervisors'])
        if ('publisher' in record): yield self._field('Publisher', record['publisher'])
        if ('source' in record): yield self._field('Source', record['source'])
        if ('date' in record): yield self._field('Publication year', record['date'])
        if ('url' in record):
            value = '<img src="http://dbiref.uvt.nl/images/edoc.gif" alt="*" border="0"/>&nbsp;<a target="_blank" href="%(url)s">%(url)s</a>' %{'url':record['url']}
            yield self._field('URL', value)
        if ('keywords' in record): yield self._field('Keyword(s)', record['keywords'])
        if ('classification' in record): yield self._field('Classification', record['classification'])
        if ('abstract' in record): yield self._field('Abstract', record['abstract'])
        if ('issn' in record): yield self._field('ISSN', record['issn'])
        if ('isbn' in record): yield self._field('ISBN', record['isbn'])
        if ('genre' in record): yield self._field('Document type', record['genre'])
        yield '\n</record>'
